package hinesm.gazoushori.gazobottun;

import java.awt.Graphics;
import java.lang.StringBuilder;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Toolkit;

/**
*このクラスは、サブクラス毎に定義した処理をボタン操作によって実行するためのクラスです。
*抽象クラスになっていることからもわかりますが、基本的に拡張して使用します。
*ボタンが操作されたかどうかは、使用先のクラスでMouseListener等を宣言する等して実行してください
**/
public abstract class GazoBottun
{


//フィールド

//このボタンが設置されるコンポーネント
private Component dependcomponent = null;

/** デフォルトの幅(100)*/
public static final int DEFAULT_WIDTH=100;
/** デフォルトの高さ(50)*/
public static final int DEFAULT_HEIGHT=50;

private int x;
private int y;
private int width;
private int height;
private int newfontdotsize;
protected String[] titlestring;
/**
ボタンの色のデフォルトの値
インスタンス作成時にはデフォルトの値が設定されます
*/
public final Color DEFAULT_BOTTUN_COLOR=new Color(0xffffffff,true);
private Color bottuncolor= new Color(DEFAULT_BOTTUN_COLOR.getRGB(),true);
/**
フレームの色のデフォルトの値
インスタンス作成時にはデフォルトの値が設定されます
*/
public final Color DEFAULT_FRAME_COLOR=new Color(0xff000000,true);
private Color framecolor = new Color(DEFAULT_FRAME_COLOR.getRGB(),true);
/**
文字の色のデフォルトの値
インスタンス作成時にはデフォルトの値が設定されます
*/
public final Color DEFAULT_TITLE_COLOR=new Color(0xff000000,true);
private Color titlecolor = new Color(DEFAULT_FRAME_COLOR.getRGB(),true);

/**
*表示文字のフォントです
*取得、設定にはgetTitleFont及びsetTitleFontを使用します。
*/
protected Font titlefont;
/**
*表示文字のフォントのデフォルトのフォント名です
*フォント名は論理フォントMONOSPACEDです
*/
public static final String DEFAULT_FONT_STRING = Font.MONOSPACED;
/**
*表示文字のフォントのデフォルトのフォントスタイルです
*フォントスタイルはプレーンスタイルです
*/
public static final int DEFAULT_FONT_STYLE = Font.PLAIN;
/**
*表示文字のフォントのデフォルトのサイズです
*フォントサイズは10ですが、コンストラクタでボタンのサイズに合わせた値を定義します
*/
public static final int DEFAULT_FONT_SIZE = 10;

/**
Tabキーによってボタンを選んで押すことができる機能の実装を検討中で、その場合そのために必要だと思われる変数です
現時点においては無意味な変数です
*/
public static int tabnow;
private int tabnum;

/**
ボタンに表示される文字です。
インスタンス作成時に指定しない場合デフォルトの文字が設定されます
*/
protected StringBuilder title;
/**
依存するコンポーネントのgetColor()で得られる色。put時に色を元に戻す際に使います
*/
public Color gc;
/**
依存するコンポーネントのgetFont()で得られるフォント。put時に元に戻す際に使います
*/
public Font gf;

private boolean frameflg;
//コンストラクタ

/**
*引数なしのコンストラクタを作成することはできません。
*必ず位置とこのボタンが設置されるコンポーネントの設定をしてください。
*幅と高さとボタンの文字の内容は任意で設定することができます。
*/
private GazoBottun()
	{}

/**
*このコンストラクタによって、作成されたインスタンスの位置が(x,y)、幅がデフォルトの100、高さがデフォルトの50に設定されます。
*ボタンに表示される文字の内容はsetTitle(String s)で設定してください。
**/
 
public GazoBottun(int x,int y,Component c)
	{
this(x,y,DEFAULT_WIDTH,DEFAULT_HEIGHT,"title",c);
	}

/**
*このコンストラクタによって、作成されたインスタンスの位置を設定します
*また、ボタンに表示される文字の内容がsに設定されます。
*/
 
public GazoBottun(int x,int y,String s,Component c)
	{
this(x,y,DEFAULT_WIDTH,DEFAULT_HEIGHT,s,c);
	}

/**
*このコンストラクタによって、作成されたインスタンスの位置が(x,y)、幅がwidth、高さがheightに設定されます。
*ボタンに表示される文字の内容はsetTitle(String s)で設定してください。
*/
 
public GazoBottun(int x,int y,int width,int height,Component c)
	{
this(x,y,width,height,"title",c);
	}

/**
*このコンストラクタによって、作成されたインスタンスの位置が(x,y)、幅がwidth、高さがheightに設定されます。
*また、ボタンの内容がsに設定されます。
*/
 
public GazoBottun(int x,int y,int width,int height,String s,Component c)
	{
setDependComponent(c);
setStetus(x,y,width,height);
setTitle(s);
setTitleFont(new Font(DEFAULT_FONT_STRING,DEFAULT_FONT_STYLE,getFontSize(difineFontDot(s,getWidth(),getHeight()))));
//getGInfo(getDependComponent().getGraphics());
setFrameFlg(true);
	}
//utillity

/**
*フォントに設定したいドット数を引数に入れることで、そのドット数に近くなるフォントのポイント値を返します
*その数値をFont(String name,int style,int size)コンストラクタのsizeに代入してください
*/
public static final int getFontSize(int d)
	{
int s;
try
		{
s=(d*72/Toolkit.getDefaultToolkit().getScreenResolution());
		}
catch(Exception e)
		{
System.out.println(e);
s = 10;
		}
return s;
	}

//difine fontdotsize
public int difineFontDot(String s,int gW,int gH)
	{
newfontdotsize=1;
if(s.length()>0)
		{
int Wpl = (gW/(s.length()));
int Hl = (gH*(s.length()));
if(Wpl<=gH)
			{
	if(Hl<=gW)
				{
newfontdotsize=gH;
				}
	else
				{
newfontdotsize=Wpl;
				}
			}
else
			{
newfontdotsize=gH;
			}
		}
else
		{
newfontdotsize=gH;
		}
return newfontdotsize;
	}

public int getFontDot()
	{
return newfontdotsize;
	}

//フィールドの設定

/**
*このボタンの設置されるコンポーネントを設定します
**/
private final void setDependComponent(Component c)
	{
dependcomponent = c;
	}

/**
*このボタンに表示される名前を設定します。
**/
 
public void setTitle(String s)
	{
if(title!=null)
		{
if(!(s.equals(title)))
			{
title.replace(0,title.length(),s);
makeString();
			}
		}
else
		{
title= new StringBuilder(s);
makeString();
		}
	}

private void makeString()
	{
titlestring=new String[title.length()];
int cou=0;
while(titlestring.length>cou)
				{
titlestring[cou]=title.substring(cou,cou+1);
cou=cou+1;
				}
	}


/**
*このボタンの位置を設定します。
*/
 
public final void setLocation(int x,int y)
	{
this.x=x;
this.y=y;
	}

/**
*このボタンの幅と高さを設定します。
*/
 
public final void setSize(int width,int height)
	{
this.width = width;
this.height = height;
	}

/**
*このボタンの位置と幅と高さを設定します。
*/
 
public final void setStetus(int x,int y,int width,int height)
	{
setLocation(x,y);
setSize(width,height);
	}

/**
引数のaの色をcに変換する。
引数がColor型の色変更メソッドに必須
*/
private static final Color translateColor(Color c,Color a)
	{
Color ret;
if(c.getRGB()!=a.getRGB())
		{
//System.out.println("c.getRGB()="+Integer.toHexString(c.getRGB()));
//System.out.println("a.getRGB()="+Integer.toHexString(a.getRGB()));
ret = new Color(c.getRGB(),true);
//System.out.println("after success a.getRGB()="+Integer.toHexString(a.getRGB()));
		}
else
		{
ret=a;
		}
return ret;
	}
/**
*このボタンの色をARGB値で設定します。
*引数のaflgがtrueの場合アルファ値をそのままに、
*falseの場合アルファ値をffとして設定します（つまり内部的には透明度が設定された色ですが、数値的には透明度0なので非透明と同じ状態にします）。
*新しいColorインスタンスを作成するので高速処理には不向きです。
*より詳細な内容が知りたい場合は、java.awt.ColorクラスのColor(int rgba,boolean hasalpha)コンストラクタを確認してください（このメソッドのソースは、ボタンの色に、java.awt.Colorクラスの新しいインスタンスを上記の引数で代入しているのみです）。
*/
 
public final void setColor(int n,boolean aflg)
	{
bottuncolor = new Color(n,aflg);
	}
/**
このボタンの色をColorインスタンスで設定します。
こちらは代入された色のsRGB値と元々のこのボタンの色のsRGB値が一致していない限り新しくインスタンスを作成することがないので比較的高速処理に向いています。
*/
public final void setColor(Color c)
	{
bottuncolor = translateColor(c,bottuncolor);
	}

/**
*このボタンの文字色をARGB値で設定します。
*対象とするColor型フィールド（文字の色を示すフィールドです）以外はsetColor(int n,boolean aflg)と同じなので詳細はそちらを確認してください。
*/
 
public final void setTitleColor(int n,boolean aflg)
	{
titlecolor = new Color(n,aflg);
	}
/**
*このボタンの文字色をARGB値で設定します。
*対象とするColor型フィールド（文字の色を示すフィールドです）以外はsetColor(COlor c)と同じなので詳細はそちらを確認してください。
*/
public final void setTitleColor(Color c)
	{
titlecolor = translateColor(c,titlecolor);
	}
/**
*このボタンの枠のをARGB値で設定します。
*対象とするColor型フィールド（枠の色を示すフィールドです）以外はsetColor(int n,boolean aflg)と同じなので詳細はそちらを確認してください。
*/
 
public final void setFrameColor(int n,boolean aflg)
	{
framecolor= new Color(n,aflg);
	}
/**
*このボタンの文字色をARGB値で設定します。
*対象とするColor型フィールド（文字の色を示すフィールドです）以外はsetColor(COlor c)と同じなので詳細はそちらを確認してください。
*/
public final void setFrameColor(Color c)
	{
framecolor = translateColor(c,framecolor);
	}
/**
*このボタンに表示される文字のフォントを設定します
*/
public final void setTitleFont(Font f)
	{
Font ff;
if(f.getSize()>getFontSize(difineFontDot(getTitle(),getWidth(),getHeight())))
		{
ff=new Font(f.getFontName(),f.getStyle(),getFontSize(difineFontDot(getTitle(),getWidth(),getHeight())));
		}
else
		{
ff=f;
		}
titlefont = ff;
	}

public void setFrameFlg(boolean b)
	{
frameflg=b;
	}
//フィールドの取得

/**
このボタンのx座標を返します
*/
 
public final int getX()
	{
return x;
	}
/**
このボタンのy座標を返します
*/
 
public final int getY()
	{
return y;
	}
/**
このボタンの幅を返します
*/
 
public final int getWidth()
	{
return width;
	}
/**
このボタンの高さを返します
*/
 
public final int getHeight()
	{
return height;
	}
/**
このボタンに表示される文字を返します
*/
 
public final String getTitle()
	{
return title.toString();
	}
/**
このボタンの設置されているコンポーネントを返します
*/

public final Component getDependComponent()
	{
return dependcomponent;
	}

/**
このボタンの色を返します
*/
 
public Color getColor()
	{
return bottuncolor;
	}
/**
このボタンに表示される文字の色を返します
*/
 
public final Color getTitleColor()
	{
return titlecolor;
	}
/**
このボタンの枠の色を返します
*/
 
public final Color getFrameColor()
	{
return framecolor;
	}
/**
*このボタンのタイトルのフォントを返します
*/
public final Font getTitleFont()
	{
return titlefont;
	}

//描画関連
/**
引数に指定したGraphicsインスタンスで現在使用しているgetFont()及びgetColor()のコピーを、
このクラスのインスタンスのフィールドに新しく作成、代入します。
これを用いることでputの時にGraphicsのsetFont()及びsetColor()で元に戻すことが可能となります。
(ただし、put()またはput(Graphics g)の実行時にはこのgetInfo(Graphics g)メソッtドを呼び出したされた時点のFont及びColorを代入するので、put()を呼び出す前にsetColor()などで色を変更していた場合、期待していた処理は行えない（つまり元に戻すのではなく別の情報を代入してしまう）可能性があります)
*/
public void getGInfo(Graphics g)
	{

if(gf==null)
		{
gf = new Font(new String(g.getFont().getName()),g.getFont().getStyle(),g.getFont().getSize());
		}
if(g.getFont().getName()!=gf.getName() || g.getFont().getStyle() != gf.getStyle() || g.getFont().getSize()!=gf.getSize())
		{
gf = new Font(new String(g.getFont().getName()),g.getFont().getStyle(),g.getFont().getSize());
		}

if(gc==null)
		{
gc=new Color(g.getColor().getRGB(),true);
		}
if(g.getColor().getRGB()!=gc.getRGB())
		{
gc = new Color(g.getColor().getRGB(),true);
		}

	}


/**
*このメソッドは、依存するコンポーネントからgetGraphicsで得られるGraphicsインスタンスにこのボタンを描画するためのクラスです。
*描画するのみなので押してもマウスを移動させても何の反応もありません。
*マウスに関する処理は他でやってください
*/
public void put()
	{
put(getDependComponent().getGraphics());
	}
/**
*このメソッドは、引数となったGraphicsインスタンスにこのボタンを描画するためのクラスです。
*描画するのみなので押してもマウスを移動させても何の反応もありません。
*マウスに関する処理は他でやってください
*引数なしのput()は自動的に依存するコンポーネントのGraphicsに対して書き込みを行います
*なので基本的にはダブルバッファリング時に別のGraphicsインスタンスに書き込むためのメソッドとして利用してください
**/

public void put(Graphics g)
	{
try
		{
getGInfo(g);
drawFrame(g);
drawTitle(g);
//元の色に戻します
g.setColor(gc);
		}
catch(Exception e)
		{
System.out.println(e);
		}
	}

public void drawFrame(Graphics g)
	{
if(frameflg)
		{
//枠の内部を描きます
g.setColor(this.getColor());
g.fillRect(x+1,y+1,width-1,height-1);
//枠を描きます
g.setColor(this.getFrameColor());
g.drawLine(x,y+1,x,y+height-1);//左辺
g.drawLine(x+1,y,x+width-1,y);//上辺
g.drawLine(x+width,y+1,x+width,y+height-1);//右辺
g.drawLine(x+1,y+height,x+width-1,y+height);//底辺
		}
	}

public void drawTitle(Graphics g)
	{
//文字を描きます
g.setFont(this.getTitleFont());
g.setColor(this.getTitleColor());
int cou=0;
if(titlestring!=null)
		{
while(titlestring.length>cou)
			{
g.drawString(titlestring[cou],x+getFontDot()*cou,y+(getHeight()*3/4));
cou=cou+1;
			}
		}
g.setFont(gf);
	}

//ボタンの内容の実行関連

/**
*このメソッドはマウスでの操作を想定して作成したものです。
*このメソッドを実行したgazoBottunインスタンスの(x,y,width,height)領域の中に(mx,my)で指定した位置が存在すればtrueを返します。
**/
 
 
public boolean containCheck(int mx,int my)
	{
return (mx>=getX() && mx<=getX()+getWidth() && my>=getY() && my<=getY()+getHeight());
	}

/**
*このメソッドはサブクラスでオーバーライドし、、
*このクラスのサブクラスで定義された処理を実行します。
**/
public abstract void perform();

/**
*このメソッドは、引数がtrueの場合、上記のperform()で定義された処理を実行します。
*このメソッドの引数にcontainCheck(int mx,int my)を用いる場合は好みによってifContainPerformを使いわけるとよいでしょう。
**/
 
public void perform(boolean b)
	{
if(b)
		{
perform();
		}
	}

/**
*このメソッドは指定された位置（x,y）が、ボタンの領域に収まっている場合perform()を実行します。
*すっきりしたコードにしたい場合にperform(containCheck(int mx,int my))と使い分けるとよいでしょう
*(ソースを見ると分かりますが中のコードは上記を実行しているのみです。)
**/
 
public void ifContainPerform(int mx,int my)
	{
perform(containCheck(mx,my));
	}
}
